@@ -51,7 +51,8 @@ class Operations(object):
         alembic.op._remove_proxy()
 
     def _foreign_key_constraint(self, name, source, referent, 
-                                    local_cols, remote_cols):
+                                    local_cols, remote_cols,
+                                    onupdate=None, ondelete=None):
         m = schema.MetaData()
         t1 = schema.Table(source, m, 
                 *[schema.Column(n, NULLTYPE) for n in local_cols])
@@ -61,7 +62,9 @@ class Operations(object):
         f = schema.ForeignKeyConstraint(local_cols, 
                                             ["%s.%s" % (referent, n) 
                                             for n in remote_cols],
-                                            name=name
+                                            name=name,
+                                            onupdate=onupdate,
+                                            ondelete=ondelete
                                             )
         t1.append_constraint(f)
 
@@ -95,7 +98,7 @@ class Operations(object):
         return schema.Column(name, type_, **kw)
 
     def _index(self, name, tablename, columns, **kw):
-        t = schema.Table(tablename, schema.MetaData(),
+        t = schema.Table(tablename or 'no_table', schema.MetaData(),
             *[schema.Column(n, NULLTYPE) for n in columns]
         )
         return schema.Index(name, *list(t.c), **kw)
@@ -315,7 +318,7 @@ class Operations(object):
 
 
     def create_foreign_key(self, name, source, referent, local_cols, 
-                                    remote_cols):
+                           remote_cols, onupdate=None, ondelete=None):
         """Issue a "create foreign key" instruction using the 
         current migration context.
 
@@ -349,12 +352,19 @@ class Operations(object):
          source table.
         :param remote_cols: a list of string column names in the
          remote table.
+        :param onupdate: Optional string. If set, emit ON UPDATE <value> when
+         issuing DDL for this constraint. Typical values include CASCADE,
+         DELETE and RESTRICT.
+        :param ondelete: Optional string. If set, emit ON DELETE <value> when
+         issuing DDL for this constraint. Typical values include CASCADE,
+         DELETE and RESTRICT.
 
         """
 
         self.impl.add_constraint(
                     self._foreign_key_constraint(name, source, referent, 
-                            local_cols, remote_cols)
+                            local_cols, remote_cols,
+                            onupdate=onupdate, ondelete=ondelete)
                 )
 
     def create_unique_constraint(self, name, source, local_cols, **kw):
@@ -502,7 +512,7 @@ class Operations(object):
             self._index(name, tablename, *columns, **kw)
         )
 
-    def drop_index(self, name):
+    def drop_index(self, name, tablename=None):
         """Issue a "drop index" instruction using the current 
         migration context.
 
@@ -510,9 +520,14 @@ class Operations(object):
         e.g.::
 
             drop_index("accounts")
+            
+        :param tablename: name of the owning table.  Some
+         backends such as Microsoft SQL Server require this.
 
         """
-        self.impl.drop_index(self._index(name, 'foo', []))
+        # need a dummy column name here since SQLAlchemy
+        # 0.7.6 and further raises on Index with no columns
+        self.impl.drop_index(self._index(name, tablename, ['x']))
 
     def drop_constraint(self, name, tablename):
         """Drop a constraint of the given name"""
@@ -591,7 +606,7 @@ class Operations(object):
         """
         return impl._literal_bindparam(None, value, type_=type_)
 
-    def execute(self, sql):
+    def execute(self, sql, execution_options=None):
         """Execute the given SQL using the current migration context.
 
         In a SQL script context, the statement is emitted directly to the 
@@ -649,9 +664,12 @@ class Operations(object):
         * Pretty much anything that's "executable" as described
           in :ref:`sqlexpression_toplevel`.
 
-
+        :param execution_options: Optional dictionary of 
+         execution options, will be passed to 
+         :meth:`sqlalchemy.engine.base.Connection.execution_options`.
         """
-        self.migration_context.impl.execute(sql)
+        self.migration_context.impl.execute(sql, 
+                    execution_options=execution_options)
 
     def get_bind(self):
         """Return the current 'bind'.
